using System.Collections.Generic;
using System.Text.Json;

namespace Adaptation.Shared.Methods;

public interface IDescription
{

    string GetEventDescription();
    List<string> GetDetailNames();
    List<string> GetHeaderNames();
    IDescription GetDisplayNames();
    List<string> GetParameterNames();
    List<string> GetPairedParameterNames();
    List<string> GetIgnoreParameterNames(Test test);
    List<string> GetNames(IFileRead fileRead, Logistics logistics);
    JsonProperty[] GetDefault(IFileRead fileRead, Logistics logistics);
    Dictionary<string, string> GetDisplayNamesJsonElement(IFileRead fileRead);
    IDescription GetDefaultDescription(IFileRead fileRead, Logistics logistics);
    List<IDescription> GetDescriptions(IFileRead fileRead, Logistics logistics, List<Test> tests, IProcessData iProcessData);

}